            <dt{% if not typedef.has_details and typedef.base_url == compound.url %} id="{{ typedef.id }}"{% endif %}>
              {% if typedef.templates != None %}
              {% set j = joiner(', ') %}
              <div class="m-doc-template">template&nbsp;&lt;<span class="m-doc-template-params{% if typedef.templates|length >= 3 %} m-doc-template-long{% endif %}">{% for t in typedef.templates %}{{ j() }}<span class="m-doc-template-param">{{ t.type }}{% if t.name %} {{ t.name }}{% endif %}{% if t.default %} = {{ t.default }}{% endif%}</span>{% endfor %}</span>&gt;</div>
              {% endif %}
              using <a href="{% if typedef.base_url != compound.url %}{{ typedef.base_url }}{% endif %}#{{ typedef.id }}" class="m-doc{% if not typedef.has_details and typedef.base_url == compound.url %}-self{% endif %}">{{ typedef.name }}</a> = {{ typedef.type }}{{ typedef.args }}{% if typedef.deprecated %} <span class="m-label m-danger">{{ typedef.deprecated }}</span>{% endif %}{% if mark_nonpublic and typedef.is_protected %} <span class="m-label m-flat m-warning">protected</span>{% endif %}{% if typedef.since %} {{ typedef.since }}{% endif %}
              {# This empty line needs to be there otherwise it's eaten #}

            </dt>
            <dd>{{ typedef.brief }}</dd>
