            <dt>
              {% if concept.templates != None %}
              {% set j = joiner(', ') %}
              <div class="m-doc-template">template&nbsp;&lt;<span class="m-doc-template-params{% if concept.templates|length >= 3 %} m-doc-template-long{% endif %}">{% for t in concept.templates %}{{ j() }}<span class="m-doc-template-param">{{ t.type }}{% if t.name %} {{ t.name }}{% endif %}{% if t.default %} = {{ t.default }}{% endif %}</span>{% endfor %}</span>&gt;</div>
              {% endif %}
              {{ concept.kind }} <a href="{{ concept.url }}" class="m-doc">{{ concept.name }}</a>{% if concept.is_protected %} <span class="m-label m-flat m-warning">protected</span>{% endif %}{% if concept.deprecated %} <span class="m-label m-danger">{{ concept.deprecated }}</span>{% endif %}{% if concept.since %} {{ concept.since }}{% endif %}

              {# the empty line is above to fix spacing #}
            </dt>
            <dd>{{ concept.brief }}</dd>
