{% extends 'base-reference.html' %}

{% block title %}{% set j = joiner(' &raquo; ') %}{% for name, _ in compound.breadcrumb %}{{ j() }}{{ name }}{% endfor %} module | {{ super() }}{% endblock %}

{% block header %}
        <h1>
          {% for name, target in compound.breadcrumb[:-1] %}
          <span class="m-breadcrumb"><a href="{{ target }}">{{ name }}</a> &raquo;</span>
          {% endfor %}
          {{ compound.breadcrumb[-1][0] }} <span class="m-thin">module</span>{% if compound.since %} {{ compound.since }}{% endif %}</h1>
{% endblock %}
