            <dt{% if not var.has_details and var.base_url == compound.url %} id="{{ var.id }}"{% endif %}>
              {% if var.templates != None %}
              {% set j = joiner(', ') %}
              <div class="m-doc-template">template&nbsp;&lt;<span class="m-doc-template-params{% if var.templates|length >= 3 %} m-doc-template-long{% endif %}">{% for t in var.templates %}{{ j() }}<span class="m-doc-template-param">{{ t.type }}{% if t.name %} {{ t.name }}{% endif %}{% if t.default %} = {{ t.default }}{% endif%}</span>{% endfor %}</span>&gt;</div>
              {% endif %}
              {%+ if var.is_static %}static {% endif %}{{ var.type }} <a href="{% if var.base_url != compound.url %}{{ var.base_url }}{% endif %}#{{ var.id }}" class="m-doc{% if not var.has_details and var.base_url == compound.url %}-self{% endif %}">{{ var.name }}</a>{% if var.deprecated %} <span class="m-label m-danger">{{ var.deprecated }}</span>{% endif %}{% if mark_nonpublic and var.is_protected %} <span class="m-label m-flat m-warning">protected</span>{% endif %}{% if var.is_constexpr %} <span class="m-label m-flat m-primary">constexpr</span>{% endif %}{% if var.since %} {{ var.since }}{% endif %}
              {# This empty line needs to be there otherwise it's eaten #}

            </dt>
            <dd>{{ var.brief }}</dd>
