{% extends 'base.html' %}

{% block main %}
        <script>
        function toggle(e) {
            e.parentElement.className = e.parentElement.className == 'm-doc-collapsible' ?
                'm-doc-expansible' : 'm-doc-collapsible';
            return false;
        }
        /* Collapse all nodes marked as such. Doing it via JS instead of
           directly in markup so disabling it doesn't harm usability. The list
           is somehow regenerated on every iteration and shrinks as I change
           the classes. It's not documented anywhere and I'm not sure if this
           is the same across browsers, so I am going backwards in that list to
           be sure. */
        var collapsed = document.getElementsByClassName("collapsed");
        for(var i = collapsed.length - 1; i >= 0; --i)
            collapsed[i].className = 'm-doc-expansible';
        </script>
{% endblock %}
