            <dt>
              {% if class.templates != None %}
              {% set j = joiner(', ') %}
              <div class="m-doc-template">template&nbsp;&lt;<span class="m-doc-template-params{% if class.templates|length >= 3 %} m-doc-template-long{% endif %}">{% for t in class.templates %}{{ j() }}<span class="m-doc-template-param">{{ t.type }}{% if t.name %} {{ t.name }}{% endif %}{% if t.default %} = {{ t.default }}{% endif %}</span>{% endfor %}</span>&gt;</div>
              {% endif %}
              {{ class.kind }} <a href="{{ class.url }}" class="m-doc">{{ class.name }}</a>{% if class.is_protected %} <span class="m-label m-flat m-warning">protected</span>{% endif %}{% if class.is_final %} <span class="m-label m-flat m-warning">final</span>{% elif class.is_virtual %} <span class="m-label m-flat m-warning">virtual</span>{% endif %}{% if class.deprecated %} <span class="m-label m-danger">{{ class.deprecated }}</span>{% endif %}{% if class.since %} {{ class.since }}{% endif %}

              {# the empty line is above to fix spacing #}
            </dt>
            <dd>{{ class.brief }}</dd>
