          <section class="m-doc-details" id="{{ enum.id }}"><div>
            <h3>
              {% if compound.templates != None %}
              <div class="m-doc-template">
                {% set j = joiner(', ') %}
                template&nbsp;&lt;<span class="m-doc-template-params{% if compound.templates|length >= 3 %} m-doc-template-long{% endif %}">{% for t in compound.templates %}{{ j() }}<span class="m-doc-template-param">{{ t.type }} {% if t.name %}{{ t.name }}{% else %}_{{ loop.index }}{% endif %}</span>{% endfor %}</span>&gt;
              </div>
              {% endif %}
              enum {% if enum.is_strong %}class {% endif %}<span class="m-doc-details-prefix">{{ prefix }}</span><a href="#{{ enum.id }}" class="m-doc-self">{{ enum.name }}</a>{% if enum.type %}: {{ enum.type }}{% endif %}{% if enum.is_protected %} <span class="m-label m-warning">protected</span>{% endif %}{% if enum.since %} {{ enum.since }}{% endif %}
              {# not sure why there needs to be this space #}

              {% if enum.include %}
              {# Template info can be only present if the enum is inside a
                 templated class, but in that case we have global include
                 information, so no need to handle case where
                 `enum.include and compound.templates != None` #}
              <div class="m-doc-include m-code m-inverted m-text-right"><span class="cp">#include</span> <a class="cpf" href="{{ enum.include[1] }}">{{ enum.include[0] }}</a></div>
              {% endif %}
            </h3>
            {% if enum.brief %}{# brief can be omitted for anonymous enums #}
            <p>{{ enum.brief }}</p>
            {% endif %}
            {% if enum.description %}
{{ enum.description }}
            {% endif %}
            {% if enum.has_value_details %}
            <table class="m-table m-fullwidth m-flat m-doc">
              <thead><tr><th style="width: 1%">Enumerators</th><th></th></tr></thead>
              <tbody>
                {% for value in enum.values %}
                <tr>
                  <td><a href="#{{ value.id }}" class="m-doc-self" id="{{ value.id }}">{{ value.name }}</a>{% if value.since %} {{ value.since }}{% endif %}</td>
                  <td>
                  {% if value.brief %}{# brief is not required for values #}
                  <p>{{ value.brief }}</p>
                  {% endif %}
                  {% if value.description %}{# it can be only brief tho #}
{{ value.description }}
                  {% endif %}
                  </td>
                </tr>
                {% endfor %}
              </tbody>
            </table>
            {% endif %}
          </div></section>
