          <section class="m-doc-details" id="{{ define.id }}"><div>
            <h3>
              {% set j = joiner(',\n              ') %}
              <span class="m-doc-wrap-bumper">#define <a href="#{{ define.id }}" class="m-doc-self">{{ define.name }}</a>{% if define.params != None %}(</span><span class="m-doc-wrap">{% for param in define.params %}{{ j() }}{{ param[0] }}{% endfor %}){% endif %}{% if define.since %} {{ define.since }}{% endif %}</span>
              {% if define.include %}
              <div class="m-doc-include m-code m-inverted m-text-right"><span class="cp">#include</span> <a class="cpf" href="{{ define.include[1] }}">{{ define.include[0] }}</a></div>
              {% endif %}
            </h3>
            {% if define.brief %}
            <p>{{ define.brief }}</p>
            {% endif %}
            {% if define.has_param_details or define.return_value %}
            <table class="m-table m-fullwidth m-flat">
              {% if define.has_param_details %}
              <thead>
                <tr><th colspan="2">Parameters</th></tr>
              </thead>
              <tbody>
                {% for name, description in define.params %}
                <tr>
                  <td{% if loop.index == 1 %} style="width: 1%"{% endif %}>{{ name }}</td>
                  <td>{{ description }}</td>
                </tr>
                {% endfor %}
              </tbody>
              {% endif %}
              {% if define.return_value %}
              <tfoot>
                <tr>
                  <th{% if not define.has_param_details %} style="width: 1%"{% endif %}>Returns</th>
                  <td>{{ define.return_value }}</td>
                </tr>
              </tfoot>
              {% endif %}
            </table>
            {% endif %}
            {% if define.description %}
{{ define.description }}
            {% endif %}
          </div></section>
