.poxy-theme-dark { /* NOTE: in vanilla m.css this is the :root element */
  /* Text properties */
  --font: 'Source Sans Pro', sans-serif;
  --code-font: 'Source Code Pro', monospace;
  --font-size: 16px;
  --code-font-size: 0.9em; /* *not* rem, so it follows surrounding font size */
  --line-height: normal;
  --paragraph-indent: 1.5rem;
  --paragraph-align: justify;
  --link-decoration: underline;
  --link-decoration-nav: none;
  --link-decoration-heading: none;
  --nav-brand-case: uppercase;
  --nav-menu-case: none;
  --nav-heading-case: uppercase;
  --nav-categories-case: lowercase;
  --landing-header-case: lowercase;
  --heading-font-weight: 600;

  /* Shapes */
  --border-radius: 0.2rem;

  /* Basics */
  --background-color: #2f363f;
  --color: #dcdcdc;
  --line-color: #405363;
  --link-color: #5b9dd9;
  --link-active-color: #a5c9ea;
  --mark-color: #2f83cc;
  --mark-background-color: #c7cf2f;
  --button-background-color: #22272e;

  /* Code */
  --code-color: #e6e6e6;
  --code-background-color: #282e36;
  --code-note-background-color: rgba(34, 39, 46, 0.5);

  /* Console */
  --console-color: #fcfcfc;
  --console-background-color: #1a1c1d;

  /* Header */
  --header-border-width: 0 0 0.25rem 0;
  --header-color: #ffffff;
  --header-breadcrumb-color: #747474; /* same as --dim-color */
  --header-background-color: #22272e;
  --header-background-color-landing: rgba(34, 39, 46, 0.75);
  --header-background-color-jumbo: rgba(34, 39, 46, 0.25);
  --header-link-color: #ffffff;
  --header-link-active-color: #a5c9ea;
  --header-link-current-color: #5b9dd9;
  --header-link-active-background-color: #292f37;
  --header-link-active-background-color-semi: rgba(41, 47, 55, 0.5);

  /* Footer */
  --footer-font-size: 0.85rem;
  --footer-color: #c5c5c5;
  --footer-background-color: #444e5c;
  --footer-link-color: #ffffff;
  --footer-link-active-color: #a5c9ea;

  /* Cover image */
  --cover-image-background-color: #0f1217;

  /* Search (used only by m-documentation.css) */
  --search-overlay-color: var(--header-background-color-landing);
  --search-background-color: var(--header-background-color);

  /* Article */
  --article-header-color: #f0f0f0;
  --article-footer-color: #c5c5c5;
  --article-heading-color: #a5c9ea;
  --article-heading-active-color: #dcdcdc;

  /* Right navigation panel */
  --navpanel-link-color: #ffffff;
  --navpanel-link-active-color: #a5c9ea;

  /* Plots */
  --plot-background-color: #34424d;
  --plot-error-color: #ffffff;

  /* Colored components */
  --default-color: #dcdcdc;
  --default-link-active-color: #a5c9ea;
  --default-filled-color: #dcdcdc;
  --default-filled-background-color: #34424d;
  --default-filled-link-color: #5b9dd9;
  --default-filled-link-active-color: #a5c9ea;

  --primary-color: #a5c9ea;
  --primary-link-active-color: #dcdcdc;
  --primary-filled-color: #2f363f;
  --primary-filled-background-color: #a5c2db;
  --primary-filled-link-color: #2a75b6;
  --primary-filled-link-active-color: #2f363f;

  --success-color: #3bd267;
  --success-link-active-color: #acecbe;
  --success-filled-color: #acecbe;
  --success-filled-background-color: #2a703f;
  --success-filled-link-color: #3bd267;
  --success-filled-link-active-color: #acecbe;

  --warning-color: #c7cf2f;
  --warning-link-active-color: #e9ecae;
  --warning-filled-color: #e9ecae;
  --warning-filled-background-color: #6d702a;
  --warning-filled-link-color: #b8bf2b;
  --warning-filled-link-active-color: #e9ecae;

  --danger-color: #cd3431;
  --danger-link-active-color: #ff9391;
  --danger-filled-color: #ff9391;
  --danger-filled-background-color: #702b2a;
  --danger-filled-link-color: #d85c59;
  --danger-filled-link-active-color: #ff9391;

  --info-color: #2f83cc;
  --info-link-active-color: #5297d7;
  --info-filled-color: #a5caeb;
  --info-filled-background-color: #2a4f70;
  --info-filled-link-color: #5297d7;
  --info-filled-link-active-color: #a5caeb;

  --dim-color: #747474;
  --dim-link-color: #acacac;
  --dim-link-active-color: #747474;
  --dim-filled-color: #747474;
  --dim-filled-background-color: #2d3236;
  --dim-filled-link-color: #acacac;
  --dim-filled-link-active-color: #747474;
  --dim-button-active-color: #acacac;
}
