/*
	NOTE FOR CONTRIBUTORS

	This file does not appear in poxy-built documentation!
	Instead a pre-processed version of it is used (in order to minimize HTTP requests).

	If you are iterating on poxy's built-in CSS using the theme_test,
	you must update the preprocessed version as part of your commit:

		poxy --update-styles

	(this will work from anywhere, CWD is not relevant)

	Additionally, don't modify any css files that are part of the bundled m.css
	package (i.e. those in ../m.css/css/). The overall goal is to leave m.css as
	'vanilla' as possible (to minimize conflicts and patching if we wish to contribute
	upstream) - if you wish to override something in m.css, poxy-overrides.css is probably
	the place to put it.
*/

/* base font */
@import url("https://fonts.googleapis.com/css?family=Source+Sans+Pro:400,400i,600,600i%7CSource+Code+Pro:400,400i,600");

/* vars for dark theme */
@import url("m-theme-dark.css");
@import url("poxy-theme-dark.css");

/* vars for light theme */
@import url("m-theme-light.css");
@import url("poxy-theme-light.css");

/* m.css */
@import url("../mcss/css/m-grid.css");
@import url("../mcss/css/m-components.css");
@import url("m-special.css");
@import url("../mcss/css/m-layout.css");
@import url("poxy-pygments.css");
@import url("../mcss/css/m-documentation.css");

/* poxy overrides */
@import url("poxy-overrides.css");
