.m-console .hll { background-color: #ffffcc }
.m-console .g-AnsiBackgroundBlack { background-color: #232627 } /* Generic.AnsiBackgroundBlack */
.m-console .g-AnsiBackgroundBlue { background-color: #1d99f3 } /* Generic.AnsiBackgroundBlue */
.m-console .g-AnsiBackgroundBrightBlack { background-color: #7f8c8d } /* Generic.AnsiBackgroundBrightBlack */
.m-console .g-AnsiBackgroundBrightBlue { background-color: #3daee9 } /* Generic.AnsiBackgroundBrightBlue */
.m-console .g-AnsiBackgroundBrightCyan { background-color: #16a085 } /* Generic.AnsiBackgroundBrightCyan */
.m-console .g-AnsiBackgroundBrightGreen { background-color: #1cdc9a } /* Generic.AnsiBackgroundBrightGreen */
.m-console .g-AnsiBackgroundBrightMagenta { background-color: #8e44ad } /* Generic.AnsiBackgroundBrightMagenta */
.m-console .g-AnsiBackgroundBrightRed { background-color: #c0392b } /* Generic.AnsiBackgroundBrightRed */
.m-console .g-AnsiBackgroundBrightWhite { background-color: #ffffff } /* Generic.AnsiBackgroundBrightWhite */
.m-console .g-AnsiBackgroundBrightYellow { background-color: #fdbc4b } /* Generic.AnsiBackgroundBrightYellow */
.m-console .g-AnsiBackgroundCyan { background-color: #1abc9c } /* Generic.AnsiBackgroundCyan */
.m-console .g-AnsiBackgroundGreen { background-color: #11d116 } /* Generic.AnsiBackgroundGreen */
.m-console .g-AnsiBackgroundMagenta { background-color: #9b59b6 } /* Generic.AnsiBackgroundMagenta */
.m-console .g-AnsiBackgroundRed { background-color: #ed1515 } /* Generic.AnsiBackgroundRed */
.m-console .g-AnsiBackgroundWhite { background-color: #fcfcfc } /* Generic.AnsiBackgroundWhite */
.m-console .g-AnsiBackgroundYellow { background-color: #f67400 } /* Generic.AnsiBackgroundYellow */
.m-console .g-AnsiBlack { color: #232627 } /* Generic.AnsiBlack */
.m-console .g-AnsiBlue { color: #1d99f3 } /* Generic.AnsiBlue */
.m-console .g-AnsiBrightBlack { color: #7f8c8d; font-weight: bold } /* Generic.AnsiBrightBlack */
.m-console .g-AnsiBrightBlue { color: #3daee9; font-weight: bold } /* Generic.AnsiBrightBlue */
.m-console .g-AnsiBrightCyan { color: #16a085; font-weight: bold } /* Generic.AnsiBrightCyan */
.m-console .g-AnsiBrightDefault { color: #ffffff; font-weight: bold } /* Generic.AnsiBrightDefault */
.m-console .g-AnsiBrightGreen { color: #1cdc9a; font-weight: bold } /* Generic.AnsiBrightGreen */
.m-console .g-AnsiBrightInvertedDefault { color: #1a1c1d; font-weight: bold } /* Generic.AnsiBrightInvertedDefault */
.m-console .g-AnsiBrightMagenta { color: #8e44ad; font-weight: bold } /* Generic.AnsiBrightMagenta */
.m-console .g-AnsiBrightRed { color: #c0392b; font-weight: bold } /* Generic.AnsiBrightRed */
.m-console .g-AnsiBrightWhite { color: #ffffff; font-weight: bold } /* Generic.AnsiBrightWhite */
.m-console .g-AnsiBrightYellow { color: #fdbc4b; font-weight: bold } /* Generic.AnsiBrightYellow */
.m-console .g-AnsiCyan { color: #1abc9c } /* Generic.AnsiCyan */
.m-console .g-AnsiGreen { color: #11d116 } /* Generic.AnsiGreen */
.m-console .g-AnsiInvertedDefault { color: #1a1c1d } /* Generic.AnsiInvertedDefault */
.m-console .g-AnsiMagenta { color: #9b59b6 } /* Generic.AnsiMagenta */
.m-console .g-AnsiRed { color: #ed1515 } /* Generic.AnsiRed */
.m-console .g-AnsiWhite { color: #fcfcfc } /* Generic.AnsiWhite */
.m-console .g-AnsiYellow { color: #f67400 } /* Generic.AnsiYellow */
.m-console .go { color: #fcfcfc } /* Generic.Output */
.m-console .gp { color: #16a085; font-weight: bold } /* Generic.Prompt */
.m-console .w { color: #fcfcfc } /* Text.Whitespace */
